`timescale 1ns / 1ps

module led_test (
         clk,		 // Input clock on development board: 50Mhz
         rst_n,	 // Input reset button on development board
         led		 // Output LED lights for controlling LEDs on the development board
       );

//===========================================================================
// PORT declarations
//===========================================================================
input clk;
input rst_n;
output led;

//Register definition
reg [31:0] timer;
reg led;


//===========================================================================
// Counter counting: cycle counting 0 ~ 2 seconds
//===========================================================================
always @(posedge clk or negedge rst_n)    //Detects rising edge of clock and falling edge of reset
  begin
    if (~rst_n)			    //Reset signal active low
      timer <= 0;			    //Clear the counter
    else if (timer == 32'd99_999_999)	    //The crystal used by the development board is 50MHz, 2 seconds count (50M * 2-1 = 99_999_999)
      timer <= 0;			    //The counter counts up to 2 seconds and the counter is cleared
    else
      timer <= timer + 1'b1;	      //Increment the counter
  end

//===========================================================================
// LEDLight control
//===========================================================================
always @(posedge clk or negedge rst_n)   //Detects rising edge of clock and falling edge of reset
  begin
    if (~rst_n)			   //Reset signal active low
      led <= 1'b1;				 //LEDLight output is high
    else if (timer == 32'd49_999_999)	   //The counter counts to 1 second,
      led <= 1'b0;			   //LEDLight up
    else if (timer == 32'd99_999_999)	   //The counter counts for 2 seconds,
      led <= 1'b1;			   //LEDOff
  end
endmodule

